@@ -23,7 +23,7 @@
 #include "config.h"
 
 #include "glib.h"
-#include "gthreadinit.h"
+#include "gthreadprivate.h"
 #include "galias.h"
 
 #if defined (__GNUC__)
@@ -495,7 +495,17 @@ g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic,
 
 #ifdef DEFINE_WITH_WIN32_INTERLOCKED
 # include <windows.h>
-gint32   
+/* Following indicates that InterlockedCompareExchangePointer is
+ * declared in winbase.h (included by windows.h) and needs to be
+ * commented out if not true. It is defined iff WINVER > 0x0400,
+ * which is usually correct but can be wrong if WINVER is set before
+ * windows.h is included.
+ */
+# if WINVER > 0x0400
+#  define HAVE_INTERLOCKED_COMPARE_EXCHANGE_POINTER
+# endif
+
+gint32
 g_atomic_int_exchange_and_add (volatile gint32 *atomic,
 			       gint32           val)
 {
@@ -514,9 +524,15 @@ g_atomic_int_compare_and_exchange (volatile gint32 *atomic,
 				   gint32           oldval,
 				   gint32           newval)
 {
+#ifndef HAVE_INTERLOCKED_COMPARE_EXCHANGE_POINTER
   return (guint32) InterlockedCompareExchange ((PVOID*)atomic, 
                                                (PVOID)newval, 
                                                (PVOID)oldval) == oldval;
+#else
+  return InterlockedCompareExchange (atomic, 
+                                     newval, 
+                                     oldval) == oldval;
+#endif
 }
 
 gboolean 
@@ -524,10 +540,14 @@ g_atomic_pointer_compare_and_exchange (volatile gpointer *atomic,
 				       gpointer           oldval,
 				       gpointer           newval)
 {
-# if GLIB_SIZEOF_VOID_P != 4 /* no 32-bit system */
-#  error "InterlockedCompareExchangePointer needed"
+# ifdef HAVE_INTERLOCKED_COMPARE_EXCHANGE_POINTER
+  return InterlockedCompareExchangePointer (atomic, newval, oldval) == oldval;
 # else
+#  if GLIB_SIZEOF_VOID_P != 4 /* no 32-bit system */
+#   error "InterlockedCompareExchangePointer needed"
+#  else
    return InterlockedCompareExchange (atomic, newval, oldval) == oldval;
+#  endif
 # endif
 }
 #endif /* DEFINE_WITH_WIN32_INTERLOCKED */
@@ -613,6 +633,15 @@ g_atomic_int_get (volatile gint *atomic)
   return result;
 }
 
+void
+g_atomic_int_set (volatile gint *atomic,
+                  gint           newval)
+{
+  g_mutex_lock (g_atomic_mutex);
+  *atomic = newval;
+  g_mutex_unlock (g_atomic_mutex);
+}
+
 gpointer
 g_atomic_pointer_get (volatile gpointer *atomic)
 {
@@ -624,27 +653,46 @@ g_atomic_pointer_get (volatile gpointer *atomic)
 
   return result;
 }
+
+void
+g_atomic_pointer_set (volatile gpointer *atomic,
+                      gpointer           newval)
+{
+  g_mutex_lock (g_atomic_mutex);
+  *atomic = newval;
+  g_mutex_unlock (g_atomic_mutex);
+}
 #endif /* G_ATOMIC_OP_MEMORY_BARRIER_NEEDED */   
 #elif defined (G_ATOMIC_OP_MEMORY_BARRIER_NEEDED)
 gint
 g_atomic_int_get (volatile gint *atomic)
 {
-  gint result = *atomic;
-
   G_ATOMIC_MEMORY_BARRIER;
+  return *atomic;
+}
 
-  return result;
+void
+g_atomic_int_set (volatile gint *atomic,
+                  gint           newval)
+{
+  *atomic = newval;
+  G_ATOMIC_MEMORY_BARRIER; 
 }
 
 gpointer
 g_atomic_pointer_get (volatile gpointer *atomic)
 {
-  gpointer result = *atomic;
-
   G_ATOMIC_MEMORY_BARRIER;
-
-  return result;
+  return *atomic;
 }   
+
+void
+g_atomic_pointer_set (volatile gpointer *atomic,
+                      gpointer           newval)
+{
+  *atomic = newval;
+  G_ATOMIC_MEMORY_BARRIER; 
+}
 #endif /* DEFINE_WITH_MUTEXES || G_ATOMIC_OP_MEMORY_BARRIER_NEEDED */
 
 #ifdef ATOMIC_INT_CMP_XCHG
@@ -694,11 +742,25 @@ gint
   return g_atomic_int_get (atomic);
 }
 
+void
+(g_atomic_int_set) (volatile gint *atomic,
+		    gint           newval)
+{
+  g_atomic_int_set (atomic, newval);
+}
+
 gpointer
 (g_atomic_pointer_get) (volatile gpointer *atomic)
 {
   return g_atomic_pointer_get (atomic);
 }
+
+void
+(g_atomic_pointer_set) (volatile gpointer *atomic,
+			gpointer           newval)
+{
+  g_atomic_pointer_set (atomic, newval);
+}
 #endif /* G_ATOMIC_OP_MEMORY_BARRIER_NEEDED */
 
 #define __G_ATOMIC_C__
