@@ -174,7 +174,7 @@ g_io_error_get_from_g_error (GIOStatus status,
  * 
  * Return value: %G_IO_ERROR_NONE if the operation was successful. 
  *
- * Deprecated: Use g_io_channel_read_chars() instead.
+ * Deprecated:2.2: Use g_io_channel_read_chars() instead.
  **/
 GIOError 
 g_io_channel_read (GIOChannel *channel, 
@@ -219,7 +219,7 @@ g_io_channel_read (GIOChannel *channel,
  * 
  * Return value:  %G_IO_ERROR_NONE if the operation was successful.
  *
- * Deprecated: Use g_io_channel_write_chars() instead.
+ * Deprecated:2.2: Use g_io_channel_write_chars() instead.
  **/
 GIOError 
 g_io_channel_write (GIOChannel  *channel, 
@@ -257,7 +257,7 @@ g_io_channel_write (GIOChannel  *channel,
  * 
  * Return value: %G_IO_ERROR_NONE if the operation was successful.
  *
- * Deprecated: Use g_io_channel_seek_position() instead.
+ * Deprecated:2.2: Use g_io_channel_seek_position() instead.
  **/
 GIOError 
 g_io_channel_seek  (GIOChannel   *channel,
@@ -320,7 +320,7 @@ g_io_channel_seek  (GIOChannel   *channel,
  * flushed, ignoring errors. The channel will not be freed until the
  * last reference is dropped using g_io_channel_unref(). 
  *
- * Deprecated: Use g_io_channel_shutdown() instead.
+ * Deprecated:2.2: Use g_io_channel_shutdown() instead.
  **/
 void
 g_io_channel_close (GIOChannel *channel)
@@ -2299,11 +2299,7 @@ g_io_channel_write_unichar    (GIOChannel   *channel,
 GQuark
 g_io_channel_error_quark (void)
 {
-  static GQuark q = 0;
-  if (q == 0)
-    q = g_quark_from_static_string ("g-io-channel-error-quark");
-
-  return q;
+  return g_quark_from_static_string ("g-io-channel-error-quark");
 }
 
 #define __G_IOCHANNEL_C__
