@@ -1464,7 +1464,7 @@ g_key_file_set_string_list (GKeyFile            *key_file,
   g_return_if_fail (list != NULL);
 
   value_list = g_string_sized_new (length * 128);
-  for (i = 0; list[i] != NULL && i < length; i++)
+  for (i = 0; i < length && list[i] != NULL; i++)
     {
       gchar *value;
 
@@ -1712,7 +1712,7 @@ g_key_file_set_locale_string_list (GKeyFile            *key_file,
   g_return_if_fail (length != 0);
 
   value_list = g_string_sized_new (length * 128);
-  for (i = 0; list[i] != NULL && i < length; i++)
+  for (i = 0; i < length && list[i] != NULL; i++)
     {
       gchar *value;
       
@@ -3471,7 +3471,7 @@ g_key_file_parse_value_as_double  (GKeyFile     *key_file,
 
   double_value = g_ascii_strtod (value, &end_of_valid_d);
 
-  if (*end_of_valid_d != '\0')
+  if (*end_of_valid_d != '\0' || end_of_valid_d == value)
     g_set_error (error, G_KEY_FILE_ERROR,
                  G_KEY_FILE_ERROR_INVALID_VALUE,
                  _("Value '%s' cannot be interpreted as a float number."), value);
